@@ -1,7 +1,7 @@
 package HTTP::Server::Simple::Mason;
 use base qw/HTTP::Server::Simple::CGI/;
 use strict;
-our $VERSION = '0.03';
+our $VERSION = '0.05';
 
 =head1 NAME
 
@@ -42,6 +42,16 @@ See L<HTTP::Server::Simple> and the documentation below.
 
 
 use HTML::Mason::CGIHandler;
+use HTML::Mason::FakeApache;
+
+use Hook::LexWrap;
+
+wrap 'HTML::Mason::FakeApache::send_http_header', pre => sub {
+    my $r = shift;
+    my $status = $r->header_out('Status') || '200 H::S::Mason OK';
+    print STDOUT "HTTP/1.0 $status\n";
+};
+
 
 sub mason_handler {
     my $self = shift;
@@ -71,20 +81,50 @@ sub handle_request {
         $cgi->path_info( $cgi->path_info . "/index.html" );
     }
 
-            print <<EOF;
-HTTP/1.0 200 OK
-Content-Type: text/html
-EOF
 
-    eval { $self->mason_handler->handle_cgi_object($cgi); };
+    eval { my $m = $self->mason_handler;
+
+        $m->handle_cgi_object($cgi) };
 }
 
+
+
 sub new_handler {
     my $self    = shift;
     my $handler = HTML::Mason::CGIHandler->new(
         $self->default_mason_config,
         $self->mason_config,
-        @_
+   # Override mason's default output method so 
+   # we can change the binmode to our encoding if
+   # we happen to be handed character data instead
+   # of binary data.
+   # 
+   # Cloned from HTML::Mason::CGIHandler
+    out_method => 
+      sub {
+            my $m = HTML::Mason::Request->instance;
+            my $r = $m->cgi_request;
+            # Send headers if they have not been sent by us or by user.
+            # We use instance here because if we store $request we get a
+            # circular reference and a big memory leak.
+                unless ($r->http_header_sent) {
+                       $r->send_http_header();
+                }
+            {
+            $r->content_type || $r->content_type('text/htm'); # Set up a default
+
+            if ($r->content_type =~ /charset=([\w-]+)$/ ) {
+                my $enc = $1;
+                binmode *STDOUT, ":encoding($enc)";
+            }
+            # We could perhaps install a new, faster out_method here that
+            # wouldn't have to keep checking whether headers have been
+            # sent and what the $r->method is.  That would require
+            # additions to the Request interface, though.
+             print STDOUT grep {defined} @_;
+            }
+        },
+        @_,
     );
 
     $handler->interp->set_escape(
